#include <stdio.h>
#include "QsEventSink.h"

//--------------------------------------------------------------------------------------
//
//--------------------------------------------------------------------------------------
CSink::CSink()
{
  m_cRef = 0;
  m_pTypeInfo = NULL;

  pAlarms		= NULL;	
  AlarmBufsize	= 1000; // 1000 Alarms are buffered
  AlarmCount	= 0;
  StoreAlarms	= FALSE;

  // allocate memory for alarm buffering
  if (AlarmBufsize > 0)
  {
		pAlarms = malloc(AlarmBufsize*sizeof(CAlarm));
		memset(pAlarms,0,AlarmBufsize*sizeof(CAlarm));
  }



  hServerInitializedEvent =  CreateEvent( NULL,//LPSECURITY_ATTRIBUTES lpEventAttributes, 
										  FALSE,//BOOL bManualReset, 
										  FALSE,//BOOL bInitialState, 
										  "ServerSync"//LPTSTR lpname ); 
										  );

  hEvaluationDoneEvent	= CreateEvent( NULL,//LPSECURITY_ATTRIBUTES lpEventAttributes, 
		 							  FALSE,//BOOL bManualReset, 
									  FALSE,//BOOL bInitialState, 
									  "Data Evaluation"//LPTSTR lpname ); 
									  );
  hDataAvailEvent		= CreateEvent( NULL,//LPSECURITY_ATTRIBUTES lpEventAttributes, 
		 							  FALSE,//BOOL bManualReset, 
									  FALSE,//BOOL bInitialState, 
									  "Data Avail"//LPTSTR lpname ); 
									  );

  // Event, Alarms occurr
  hRedAlertEvent		= CreateEvent(	NULL,//LPSECURITY_ATTRIBUTES lpEventAttributes, 
		 								FALSE,//BOOL bManualReset, 
										FALSE,//BOOL bInitialState, 
										"Red Alert"//LPTSTR lpname ); 
									  );

  Owner		= 0;
  NumOfChar = 0;

  InitializeCriticalSection(&CriticalSectionAlarmListAccess); 
  Init();
}

CSink::~CSink()
{

    EnterCriticalSection(&CriticalSectionAlarmListAccess); 
	__try 
	{
		if ((AlarmBufsize > 0) && 
			(pAlarms != NULL))
		{
			free(pAlarms);
		}
	}
	__finally
	{
		LeaveCriticalSection(&CriticalSectionAlarmListAccess); 
	}
    
	
	// Release resources used by the critical section object.
    DeleteCriticalSection(&CriticalSectionAlarmListAccess);

	
	CloseHandle(hRedAlertEvent);
	CloseHandle(hServerInitializedEvent);
	CloseHandle(hEvaluationDoneEvent);
	CloseHandle(hDataAvailEvent);
}
	
	// IUnknown
ULONG CSink::AddRef()
{
	return ++m_cRef;
}

ULONG CSink::Release()
{
	if(--m_cRef != 0)
		return m_cRef;
	delete this;
	return 0;
}

HRESULT CSink::QueryInterface(REFIID riid, void** ppv)
{
	if(riid == IID_IUnknown)
	{
		*ppv = (IUnknown*)this;
	}
	else if(riid == IID_IDispatch)
	{
		*ppv = (IDispatch*)this;
	}
	else if(riid == IID_IQsEvents)
	{
		//cout << "Client: CSink::QueryInterface() for IOutGoing" << endl;
		*ppv = (IQsEvents*)this;
	}
	else 
	{
		*ppv = NULL;
		return E_NOINTERFACE;
	}
	AddRef();
	return S_OK;
}



HRESULT CSink::GetTypeInfoCount(UINT* pCountTypeInfo)
{
	*pCountTypeInfo = 1;
	return S_OK;
}

HRESULT CSink::GetTypeInfo(UINT iTypeInfo, LCID lcid, ITypeInfo** ppITypeInfo)
{
	*ppITypeInfo = NULL;
	if(iTypeInfo != 0)
		return DISP_E_BADINDEX;
	m_pTypeInfo->AddRef();
	*ppITypeInfo = m_pTypeInfo;
	return S_OK;
}

HRESULT CSink::GetIDsOfNames(REFIID riid, LPOLESTR* rgszNames, UINT cNames, LCID lcid, DISPID* rgDispId)
{
	if(riid != IID_NULL)
		return DISP_E_UNKNOWNINTERFACE;
	return DispGetIDsOfNames(m_pTypeInfo, rgszNames, cNames, rgDispId);
}

HRESULT CSink::Invoke(DISPID dispIdMember, REFIID riid, LCID lcid, WORD wFlags, DISPPARAMS* pDispParams, VARIANT* pVarResult, EXCEPINFO* pExcepInfo, UINT* puArgErr)
{
	if(riid != IID_NULL)
		return DISP_E_UNKNOWNINTERFACE;
	return DispInvoke(this, m_pTypeInfo, dispIdMember, wFlags, pDispParams, pVarResult, pExcepInfo, puArgErr); 
}

BOOL CSink::Init(void)
{
	ITypeLib* pTypeLib;
	if(FAILED(LoadRegTypeLib(LIBID_Qs_stat, 1, 0, LANG_NEUTRAL, &pTypeLib)))
		return false;
	HRESULT hr = pTypeLib->GetTypeInfoOfGuid(IID_IQsEvents, &m_pTypeInfo);
	pTypeLib->Release();
	if(FAILED(hr))
		return false;
	return true;
}

//IQSEvents
HRESULT CSink::OnGetAliveState( /* [out] */ long* state)
{
  *state = 1;// I am still alive
  return S_OK;
}

HRESULT CSink::OnDataAvail( /* [out] */  long NumOfChars )
{
  // todo: add actions if data avail

  // 1. reset the alarm buffer
  // 2. store the number of loaded characterstics
  
  AlarmCount = 0;
  memset(pAlarms,0,AlarmBufsize*sizeof(CAlarm));

//  this->NumOfChar = NumOfChars;
//  SetEvent(hDataAvailEvent);
  PostMessage(Owner, RegisterWindowMessage("MSG_OnDataAvail"),0,NumOfChars);
  return S_OK;
}

HRESULT CSink::OnDataAvailExt( /*[in] */ long handle,
							   /*[out] */ long NumOfChars)
{
// choose only one DataAvail Event  
//  AlarmCount = 0;
//  memset(pAlarms,0,AlarmBufsize*sizeof(CAlarm));
	this->NumOfChar = NumOfChars;
	SetEvent(hDataAvailEvent);
	PostMessage(Owner, RegisterWindowMessage("MSG_OnDataAvailExt"),handle,NumOfChars);
	return S_OK;
}

HRESULT CSink::OnQSSTATInitialized()
{
  // todo: add actions when server has started
	SetEvent(hServerInitializedEvent);
	PostMessage(Owner, RegisterWindowMessage("MSG_OnQSSTATInitialized"),0,0);
	return S_OK;
}

HRESULT CSink::OnEvaluationDone()
{
	// todo Add code, when evaluation is done
//  SetEvent(hEvaluationDoneEvent);
	PostMessage(Owner, RegisterWindowMessage("MSG_OnEvaluationDone"),0,0);
	return S_OK;
}

HRESULT CSink::OnEvaluationDoneExt( /*[in]*/ long handle )
{
  // todo Add code, when evaluation is done
	SetEvent(hEvaluationDoneEvent);
	PostMessage(Owner, RegisterWindowMessage("MSG_OnEvaluationDoneExt"),handle,0);
	return S_OK;
}

HRESULT CSink::OnError(/* [in] */ long ErrorCode)
{
	// todo Add code, when error occured
	return S_OK;
}
    
HRESULT CSink::OnSystemAlarm(/* [in] */ unsigned long AlarmCode )
{
	MessageBox(NULL, "System Alarm" , "QS Alarm Event",MB_OK);
	return S_OK;
}

HRESULT CSink::OnSystemAlarmExt( /*[in] */ long handle, 
								 /*[out] */ unsigned long AlarmCode )
{
//  MessageBox(NULL, "System Alarm" , "QS Alarm Event",MB_OK);
  return S_OK;
}


CAlarm* NewAlarm(	unsigned long AlarmEW1, 
					unsigned long AlarmEW2, 
					unsigned long AlarmQRK_l, 
					unsigned long AlarmQRK_s, 
					unsigned long AlarmCP, 
					unsigned long AlarmNOSPC, 
					long PartNr, 
					long CharNr, 
					long ValueNr, 
					long SampleNr)
{
	
	CAlarm*	pCurrAlarm	= new(CAlarm);

	pCurrAlarm->AlarmNOSPC	= AlarmNOSPC;
	pCurrAlarm->AlarmCP		= AlarmCP;
	pCurrAlarm->AlarmEW1	= AlarmEW1;
	pCurrAlarm->AlarmEW2	= AlarmEW2;
	pCurrAlarm->AlarmQRK_l	= AlarmQRK_l;
	pCurrAlarm->AlarmQRK_s	= AlarmQRK_s;
	pCurrAlarm->PartNr		= PartNr;
	pCurrAlarm->CharNr		= CharNr;
	pCurrAlarm->ValueNr		= ValueNr;
	pCurrAlarm->SampleNr	= SampleNr;
	return pCurrAlarm;

}

HRESULT CSink::OnStatisticalAlarm( /* [in] */ unsigned long Alarm_EW_1,
									    /* [in] */ unsigned long Alarm_EW_2,
										/* [in] */ unsigned long Alarm_QRK_L,
										/* [in] */ unsigned long Alarm_QRK_S,
										/* [in] */ unsigned long Alarm_CP,
										/* [in] */ unsigned long Alarm_NO_SPC,
										/* [in] */ long Part_Nr,
										/* [in] */ long Char_Nr,
										/* [in] */ long Value_Nr,
										/* [in] */ long Sample_Nr)
{

	if (StoreAlarms = TRUE)
	{
		SetAlarm(	Alarm_EW_1,Alarm_EW_2,Alarm_QRK_L, Alarm_QRK_S,
					Alarm_CP, Alarm_NO_SPC,
					Part_Nr, Char_Nr, Value_Nr, Sample_Nr);
	};

	SetEvent(hRedAlertEvent);

	CAlarm* pNewAlarm = NewAlarm(	Alarm_EW_1,Alarm_EW_2,Alarm_QRK_L, Alarm_QRK_S,
									Alarm_CP, Alarm_NO_SPC,
									Part_Nr, Char_Nr, Value_Nr, Sample_Nr);

	PostMessage(Owner, RegisterWindowMessage("MSG_OnStatisticalAlarm"),0,(LPARAM)pNewAlarm);

	return S_OK;
}       

HRESULT CSink::OnStatisticalAlarmExt( /*[in] */ long handle,
									  /*[in] */ unsigned long Alarm_EW_1,
									  /*[in] */ unsigned long Alarm_EW_2,
									  /*[in] */ unsigned long Alarm_QRK_L,
									  /*[in] */ unsigned long Alarm_QRK_S,
									  /*[in] */ unsigned long Alarm_CP,
									  /*[in] */ unsigned long Alarm_NO_SPC,
									  /*[in] */ long Part_Nr,
									  /*[in] */ long Char_Nr, 
									  /*[in] */ long Value_Nr, 
									  /*[in] */ long Sample_Nr)
{
	CAlarm* pNewAlarm = NewAlarm(	Alarm_EW_1,Alarm_EW_2,Alarm_QRK_L, Alarm_QRK_S,
									Alarm_CP, Alarm_NO_SPC,
									Part_Nr, Char_Nr, Value_Nr, Sample_Nr);
	PostMessage(Owner, RegisterWindowMessage("MSG_OnStatisticalAlarmExt"),handle,(LPARAM)pNewAlarm);
	return S_OK;
}    


HRESULT CSink::OnStatisticalAlarmExtVA( /* [in] */ long handle,
										/* [in] */ VARIANT Alarm_EW_1,
										/* [in] */ VARIANT Alarm_EW_2,
										/* [in] */ VARIANT Alarm_QRK_L,
										/* [in] */ VARIANT Alarm_QRK_S,
								        /* [in] */ VARIANT Alarm_CP,
										/* [in] */ VARIANT Alarm_NO_SPC,
										/* [in] */ long Part_Nr,
										/* [in] */ long Char_Nr,
									    /* [in] */ long Value_Nr,
										/* [in] */ long Sample_Nr)
{
	return S_OK;
}  

// IUpload messages (not used)
HRESULT __stdcall CSink::OnUploadProgress(	/* [in] */ long UploadChildHandle,
											/* [in] */ long UploadState,
											/* [in] */ BSTR filename,
											/* [in] */ long FileNo,
											/* [in] */ long NumOfFiles,
											/* [in] */ long PartNo,
											/* [in] */ BSTR PartKey,
											/* [in] */ long CharNo,
											/* [in] */ BSTR CharKey,
											/* [in] */ long NumOfChars,
											/* [in] */ long ValueNo,
											/* [in] */ long NumOfValues,
											/* [in] */ long ErrorNo,
											/* [in] */ BSTR ErrorMsg,
											/* [in] */ long TimeStamp)
{
  MessageBox(NULL,"Upload Progress" , "QS Alarm Event",MB_OK);
  return S_OK;
}



void CSink::SetAlarm(unsigned long AlarmEW1, 
					 unsigned long AlarmEW2, 
					 unsigned long AlarmQRK_l, 
					 unsigned long AlarmQRK_s, 
					 unsigned long AlarmCP, 
					 unsigned long AlarmNOSPC, 
					 long PartNr, 
					 long CharNr, 
					 long ValueNr, 
					 long SampleNr)
{
	//---------------------------------------------------------------------
	// write the new alarm to the buffer
	// increment no of pending alarms
	//---------------------------------------------------------------------

	CAlarm*	pCurrAlarms;
	void*	pHelp;


	// CS
    EnterCriticalSection(&CriticalSectionAlarmListAccess); 

	__try 
	{
		if ( AlarmCount >= AlarmBufsize) 
		{
			// buffer overrun, clear the olfd buffer and restart
			memset(pAlarms,0,AlarmBufsize*sizeof(CAlarm));
			AlarmCount = 0;

		}
		if (AlarmCount >= 0)
		{
			pHelp = (void*) ((long)pAlarms + AlarmCount*sizeof(CAlarm));
			pCurrAlarms  = (CAlarm*)pHelp;
			pCurrAlarms->AlarmNOSPC	= AlarmNOSPC;
			pCurrAlarms->AlarmCP	= AlarmCP;
			pCurrAlarms->AlarmEW1	= AlarmEW1;
			pCurrAlarms->AlarmEW2	= AlarmEW2;
			pCurrAlarms->AlarmQRK_l	= AlarmQRK_l;
			pCurrAlarms->AlarmQRK_s	= AlarmQRK_s;
			pCurrAlarms->PartNr		= PartNr;
			pCurrAlarms->CharNr		= CharNr;
			pCurrAlarms->ValueNr	= ValueNr;
			pCurrAlarms->SampleNr	= SampleNr;
			AlarmCount += 1;
		}
	}
	__finally 
	{
		LeaveCriticalSection(&CriticalSectionAlarmListAccess);
	}
}	


BOOL CSink::GetAlarm( unsigned long &AlarmEW1, 
					  unsigned long &AlarmEW2, 
					  unsigned long &AlarmQRK_l, 
					  unsigned long &AlarmQRK_s, 
					  unsigned long &AlarmCP, 
					  unsigned long &AlarmNOSPC, 
					  long &PartNr, 
					  long &CharNr, 
					  long &ValueNr, 
					  long &SampleNr)
{
	//---------------------------------------------------------------------
	// read the last occurred alarm from the alarm buffer 
	// decrement no of pending alarms
	//---------------------------------------------------------------------

	
	CAlarm *pCurrAlarms;
	BOOL	rc = FALSE;

	// CS
    EnterCriticalSection(&CriticalSectionAlarmListAccess); 

	__try 
	{
		if ((AlarmCount > 0) && (AlarmCount <= AlarmBufsize))
		{
			pCurrAlarms = (CAlarm*) ((long)pAlarms + (AlarmCount-1)*sizeof(CAlarm));
			AlarmEW1	= pCurrAlarms->AlarmEW1;
			AlarmEW2	= pCurrAlarms->AlarmEW2;
			AlarmQRK_l	= pCurrAlarms->AlarmQRK_l;
			AlarmQRK_s	= pCurrAlarms->AlarmQRK_s;
			AlarmCP		= pCurrAlarms->AlarmCP;
			AlarmNOSPC	= pCurrAlarms->AlarmNOSPC;
			PartNr		= pCurrAlarms->PartNr;
			CharNr		= pCurrAlarms->CharNr;
			ValueNr		= pCurrAlarms->ValueNr;
			SampleNr	= pCurrAlarms->SampleNr;
		
			AlarmCount -= 1;
			rc = TRUE; // results are valid
		}
		else
		{
			rc = FALSE;// invalid access
		}
	}
	__finally 
	{
		LeaveCriticalSection(&CriticalSectionAlarmListAccess);
		return rc;
	}

}


HRESULT CSink::OnQueryOpen(	/* [in] */ long handle,
							/* [in] */ long QueryHandle)
{
  /*nothing to do*/
  return S_OK;
}


//[id(0x0000000F)]
HRESULT CSink::OnApplicationClose(	/* [in, out]  */ long * Accept )
{
	return S_OK;
}
//[id(0x00000010)]
HRESULT CSink::OnDataSourceBeforeClose(	/* [in] */ long handle, 
										/* [in] */ long DataSourceType, 
										/* [in] */ BSTR Name, 
										/* [in, out] */  long * Accept )
{
	return S_OK;
}



