#ifndef __CSINK__
#define __CSINK__

#include "qs_stat.h"
#include "structures.h"

//---------------------------------------------------------------------------
// Qs-STAT connection point -----------Sink---------
//---------------------------------------------------------------------------
class CSink : public IQsEvents
{
private:
	long m_cRef;
	ITypeInfo* m_pTypeInfo;

	//-----------------------------------------------------------------------
	// Buffer for occurred Alarms
	//-----------------------------------------------------------------------
	void*	pAlarms;// cryptic pointer to an alarm array
	int		AlarmBufsize; // size of Alarmbuffer/ Nr. of buffered Alarms
	int		AlarmCount;
	CRITICAL_SECTION CriticalSectionAlarmListAccess; 
	void SetAlarm(	unsigned long AlarmEW1, unsigned long AlarmEW2, 
					unsigned long AlarmQRK_l, unsigned long AlarmQRK_s, 
					unsigned long AlarmCP, unsigned long AlarmNOSPC, 
					long PartNr, long CharNr, long ValueNr, long SampleNr);

public:

	//----------------------------------------------------------------------
	// qs-STAT Events
	//----------------------------------------------------------------------
	// Event for startup synchronisation with the server
	HANDLE hServerInitializedEvent;
	// Event, when data is evaluated
	HANDLE hEvaluationDoneEvent;
	// Event, when data is loaded
	HANDLE hDataAvailEvent;

	// Event, Alarms occurr
	HANDLE	hRedAlertEvent;

	BOOL StoreAlarms;

	//----------------------------------------------------------------------
	HWND	Owner;
	long NumOfChar;

	//----------------------------------------------------------------------
	CSink();
	~CSink();
	
	BOOL Init();

	BOOL GetAlarm( unsigned long &AlarmEW1, unsigned long &AlarmEW2, unsigned long &AlarmQRK_l, unsigned long &AlarmQRK_s, unsigned long &AlarmCP, unsigned long &AlarmNOSPC, long &PartNr, long &CharNr, long &ValueNr, long &SampleNr);
	
	// IUnknown
	ULONG __stdcall AddRef();
	ULONG __stdcall Release();
	HRESULT _stdcall QueryInterface(REFIID riid, void** ppv);

	// IDispatch
    HRESULT _stdcall GetTypeInfoCount( 
            /* [out] */ UINT  *pctinfo);
        
    HRESULT _stdcall GetTypeInfo( 
            /* [in] */ UINT iTInfo,
            /* [in] */ LCID lcid,
            /* [out] */ ITypeInfo *__RPC_FAR *ppTInfo);

        
    HRESULT _stdcall GetIDsOfNames( 
            /* [in] */ REFIID riid,
            /* [size_is][in] */ LPOLESTR  *rgszNames,
            /* [in] */ UINT cNames,
            /* [in] */ LCID lcid,
            /* [size_is][out] */ DISPID  *rgDispId);
        
    HRESULT _stdcall Invoke( 
            /* [in] */ DISPID dispIdMember,
            /* [in] */ REFIID riid,
            /* [in] */ LCID lcid,
            /* [in] */ WORD wFlags,
            /* [out][in] */ DISPPARAMS  *pDispParams,
            /* [out] */ VARIANT  *pVarResult,
            /* [out] */ EXCEPINFO  *pExcepInfo,
            /* [out] */ UINT  *puArgErr);
       
	//IQSEvents
	HRESULT __stdcall OnGetAliveState( /* [out] */ long* state);
	HRESULT __stdcall OnDataAvail( /* [in] */ long NumOfChars );
	HRESULT __stdcall OnDataAvailExt( /*[in] */ long handle,
									  /*[in] */ long NumOfChars );
	HRESULT __stdcall OnQSSTATInitialized();
	HRESULT __stdcall OnEvaluationDone();
	HRESULT __stdcall OnEvaluationDoneExt( /*[in]*/ long handle );
	HRESULT __stdcall OnError( /* [in] */ long ErrorCode);
	// Alarms
	HRESULT __stdcall OnSystemAlarm( /* [in] */ unsigned long AlarmCode);
	HRESULT __stdcall OnSystemAlarmExt( /*[in] */ long handle, 
										/*[in] */ unsigned long AlarmCode );
	HRESULT __stdcall OnStatisticalAlarm( /* [in] */ unsigned long Alarm_EW_1,
									      /* [in] */ unsigned long Alarm_EW_2,
										  /* [in] */ unsigned long Alarm_QRK_L,
										  /* [in] */ unsigned long Alarm_QRK_S,
										  /* [in] */ unsigned long Alarm_CP,
										  /* [in] */ unsigned long Alarm_NO_SPC,
										  /* [in] */ long Part_Nr,
										  /* [in] */ long Char_Nr,
										  /* [in] */ long Value_Nr,
										  /* [in] */ long Sample_Nr);
   HRESULT __stdcall OnStatisticalAlarmExt( /*[in] */ long handle,
										    /*[in] */ unsigned long Alarm_EW_1,
											/*[in] */ unsigned long Alarm_EW_2,
											/*[in] */ unsigned long Alarm_QRK_L,
											/*[in] */ unsigned long Alarm_QRK_S,
											/*[in] */ unsigned long Alarm_CP,
											/*[in] */ unsigned long Alarm_NO_SPC,
											/*[in] */ long Part_Nr,
											/*[in] */ long Char_Nr, 
											/*[in] */ long Value_Nr, 
											/*[in] */ long Sample_Nr);
   HRESULT __stdcall OnStatisticalAlarmExtVA( /* [in] */ long handle,
											  /* [in] */ VARIANT Alarm_EW_1,
											  /* [in] */ VARIANT Alarm_EW_2,
											  /* [in] */ VARIANT Alarm_QRK_L,
											  /* [in] */ VARIANT Alarm_QRK_S,
								              /* [in] */ VARIANT Alarm_CP,
											  /* [in] */ VARIANT Alarm_NO_SPC,
											  /* [in] */ long Part_Nr,
											  /* [in] */ long Char_Nr,
									          /* [in] */ long Value_Nr,
											  /* [in] */ long Sample_Nr);
   // IUpload messages (not used)
   HRESULT __stdcall OnUploadProgress(	/* [in] */ long UploadChildHandle,
										/* [in] */ long UploadState,
										/* [in] */ BSTR filename,
										/* [in] */ long FileNo,
										/* [in] */ long NumOfFiles,
										/* [in] */ long PartNo,
										/* [in] */ BSTR PartKey,
										/* [in] */ long CharNo,
										/* [in] */ BSTR CharKey,
										/* [in] */ long NumOfChars,
										/* [in] */ long ValueNo,
										/* [in] */ long NumOfValues,
										/* [in] */ long ErrorNo,
										/* [in] */ BSTR ErrorMsg,
										/* [in] */ long TimeStamp);
	HRESULT __stdcall OnQueryOpen(	/* [in] */ long handle,
									/* [in] */ long QueryHandle);

    //[id(0x0000000F)]
    HRESULT __stdcall OnApplicationClose(	/* [in, out]  */ long * Accept );
    //[id(0x00000010)]
    HRESULT __stdcall OnDataSourceBeforeClose(	/* [in] */ long handle, 
												/* [in] */ long DataSourceType, 
												/* [in] */ BSTR Name, 
												/* [in, out] */  long * Accept );

};
#endif